
import datetime, time, pymongo

COUNTER = "COUNTER"
GAUGE = "GAUGE"

class TimeType:
    def __init__(self,num,name,maxMinutes):
        self.num = num
        self.name = name
        self.maxMinutes = maxMinutes

    def isMinute(self):
        return self == TimeType.MINUTE

    def isHourly(self):
        return self == TimeType.HOURLY

    def isDaily(self):
        return self == TimeType.DAILY


    def __str__(self):
        return self.name

TimeType.MINUTE = TimeType(1,"minute",60*36)
TimeType.HOURLY = TimeType(2,"hourly",60*24*7)
TimeType.DAILY = TimeType(3,"daily",60*24*7*10)

TimeType.ALL = [ TimeType.MINUTE , TimeType.HOURLY , TimeType.DAILY ]

class RRD:
    
    def __init__(self,mongo_host="localhost",mongo_db="rrd",mongo_collection="rrd_data"):
        self.conn = pymongo.Connection( mongo_host )
        self.db = self.conn[mongo_db]
        self.collection = self.db[mongo_collection]
        self.idents = self.db["idents"]

    def _getDay( self, when ):
        return "%0.2d%0.2d%0.2d" % ( when.year , when.month , when.day )

    def storeAll( self, ident, dict, group=None, when=None ):
        for x in dict:
            self.store( ident, x, dict[x] , group=group, when=when )

    def store(self, ident, name, value, group=None, when=None ):
        """
        value: number
        collection: pymongo.Collection
        """

        if when is None:
            when = datetime.datetime.now()

        day = self._getDay( when )
        hour = "%0.2d" % when.hour
        time = "%0.2d%0.2d" % ( when.hour , when.minute)

        # gen key
        key = ident + "-" + day + "-"
        if group:
            key += group + "-"
        key += name

        # gen $set
        toSet = { "day" : day , 
                  "name" : name , "group" : group ,
                  "last" : value ,
                  "ident" : ident }

        # gen $inc
        toInc = { "daily.num" : 1 , "daily.total" : value ,
                  "hourly.%s.num" % hour: 1 , "hourly.%s.total" % hour : value ,
                  "minute.%s.num" % time : 1 , "minute.%s.total" % time : value }
        
        # do update
        self.collection.update( { "_id" : key } ,{ "$set" : toSet , "$inc" : toInc } , upsert=True )
        
        # update idents
        if group:
            t = { "group" : group }
        else:
            t = { "name" : name }
            
        self.idents.update( { "_id" : ident } , { "$set" : { "lastPing" : datetime.datetime.now() } , "$addToSet" : { "types" : t } } , upsert=True )
    
    def getIdents(self):
        lst = []
        for doc in self.idents.find():
            lst.append( doc["_id"] )
        return lst

    def getTypes(self,ident):
        x = self.idents.find_one( { "_id" : ident } )
        if not x:
            return None
        return x["types"]


    def parseTime(self,x):
        if len(x) == 13:
            d = datetime.datetime.strptime( x ,"%Y%m%d-%H%M" )
        elif len(x) == 11:
            d = datetime.datetime.strptime( x ,"%Y%m%d-%H" )
        elif len(x) == 8:
            d = datetime.datetime.strptime( x ,"%Y%m%d" )
        else:
            raise Exception( "can't parse: " + x )
        return time.mktime( d.timetuple() )        

    def diffMinutes(self,a,b):
        x = self.parseTime(a)
        y = self.parseTime(b)
        return (y-x)/60
        
    def getDataFromQuery( self, ident, timeType, name=None, group=None, isCounter=False ):
        if not name and not group:
            raise Exception( "need name or group or both" )
        
        query = {}
        if name:
            query["name"] = name
        if group:
            query["group"] = group
        
        query["ident"] = ident
            
        first = True

        data = {}

        def getValue( d ):
            return d["total"] / d["num"]

        for doc in self.collection.find( query ):

            #xtract basics
                
            keyname = ""
            if "group" in doc and doc["group"]:
                keyname += doc["group"] + "."
            keyname += doc["name"]
                
            day = doc["day"]
            
            #make sure list is there
            if keyname not in data:
                data[keyname] = []
            lst = data[keyname]

               
            # add actual data to list
            if timeType == TimeType.DAILY:
                lst.append( ( day , getValue( doc["daily"] ) ) )
            else:
                t = timeType.name
                    
                for x in doc[t]:
                    lst.append( ( day + "-" + x , getValue( doc[t][x] ) ) )
                
        for x in data:
            data[x].sort( lambda x,y: cmp( x[0] , y[0] ) )

        if isCounter:
            for x in data:
                old = data[x]
                prev = old.pop(0)
                new = []
                for p in old:
                    d = self.diffMinutes(prev[0],p[0])
                    if timeType.isHourly():
                        d = d / 60
                    elif timeType.isDaily():
                        d = d / ( 60 * 24 )

                    v = ( p[1] - prev[1] )
                    if d > 0:
                        v = v / d 
                    
                    if timeType.isMinute():
                        v = v / 60
                    elif timeType.isHourly():
                        v = v / 3600
                    elif timeType.isDaily():
                        v = v / 86400
                    else:
                        raise Exception("what")

                    if v >= 0:
                        new.append( ( p[0] , v ) )
                    prev = p
                    
                while len(new) > 2 and self.diffMinutes( new[0][0] , new[len(new)-1][0] ) > timeType.maxMinutes:
                    new.pop(0)

                data[x] = new
                    
        return data

if __name__ == "__main__":
    # this is for testing really

    rrd = RRD()
    for x in rrd.getDataFromQuery( "ny187.shopwiki.com" , group="opcounters" , name="query", timeType=TimeType.HOURLY, isCounter=True )["opcounters.query"]:
        print( x )
    #rrd.store( "query" , 5  )
    
    

